@@ -259,11 +259,16 @@ static ap_inline int buff_read(BUFF *fb, void *buf, int nbyte)
     }
     else
 	rv = ap_read(fb, buf, nbyte);
+#elif defined (BEOS)
+    if (fb->flags & B_SOCKET) {
+    rv = recv(fb->fd_in, buf, nbyte, 0);
+    } else
+    rv = ap_read(fb,buf,nbyte);
 #elif defined(TPF)
     fd_set fds;
     struct timeval tv;
 
-    tpf_process_signals();
+    ap_check_signals();
     if (fb->flags & B_SOCKET) {
         alarm(rv = alarm(0));
         FD_ZERO(&fds);
@@ -271,11 +276,10 @@ static ap_inline int buff_read(BUFF *fb, void *buf, int nbyte)
         tv.tv_sec = rv+1;
         tv.tv_usec = 0;
         rv = ap_select(fb->fd_in + 1, &fds, NULL, NULL, &tv);
-        if (rv < 1) {
-            tpf_process_signals();
-            return(rv);
-        }
+        if (rv > 0)
+            rv = ap_read(fb, buf, nbyte);
     }
+    else
     rv = ap_read(fb, buf, nbyte);
 #else
     rv = ap_read(fb, buf, nbyte);
@@ -316,6 +320,11 @@ static ap_inline int buff_write(BUFF *fb, const void *buf, int nbyte)
     }
     else
 	rv = ap_write(fb, buf, nbyte);
+#elif defined(BEOS)
+    if(fb->flags & B_SOCKET) {
+    rv = send(fb->fd, buf, nbyte, 0);
+    } else 
+    rv = ap_write(fb, buf,nbyte);
 #else
     rv = ap_write(fb, buf, nbyte);
 #endif /* WIN32 */
@@ -682,6 +691,7 @@ static int read_with_errors(BUFF *fb, void *buf, int nbyte)
     return rv;
 }
 
+
 /*
  * Read up to nbyte bytes into buf.
  * If fewer than byte bytes are currently available, then return those.
@@ -1437,6 +1447,16 @@ API_EXPORT(int) ap_bclose(BUFF *fb)
 	    rc3 = 0;
     }
     else {
+#elif defined(BEOS)
+    if (fb->flags & B_SOCKET) {
+	rc2 = ap_pclosesocket(fb->pool, fb->fd);
+	if (fb->fd_in != fb->fd) {
+	    rc3 = ap_pclosesocket(fb->pool, fb->fd_in);
+	}
+	else {
+	    rc3 = 0;
+	}
+    } else {
 #endif
 	rc2 = ap_pclosef(fb->pool, fb->fd);
 	if (fb->fd_in != fb->fd) {
@@ -1445,7 +1465,7 @@ API_EXPORT(int) ap_bclose(BUFF *fb)
 	else {
 	    rc3 = 0;
 	}
-#ifdef WIN32
+#if defined(WIN32) || defined (BEOS)
     }
 #endif
 
