@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1996-1998 The Apache Group.  All rights reserved.
+ * Copyright (c) 1996-1999 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,19 +56,23 @@
  */
 
 #include "mod_proxy.h"
+
+#define CORE_PRIVATE
+
 #include "http_log.h"
 #include "http_vhost.h"
+#include "http_request.h"
 
 /* Some WWW schemes and their default ports; this is basically /etc/services */
 /* This will become global when the protocol abstraction comes */
 static struct proxy_services defports[] =
 {
-    {"ftp", DEFAULT_FTP_PORT},
-    {"gopher", DEFAULT_GOPHER_PORT},
     {"http", DEFAULT_HTTP_PORT},
+    {"ftp", DEFAULT_FTP_PORT},
+    {"https", DEFAULT_HTTPS_PORT},
+    {"gopher", DEFAULT_GOPHER_PORT},
     {"nntp", DEFAULT_NNTP_PORT},
     {"wais", DEFAULT_WAIS_PORT},
-    {"https", DEFAULT_HTTPS_PORT},
     {"snews", DEFAULT_SNEWS_PORT},
     {"prospero", DEFAULT_PROSPERO_PORT},
     {NULL, -1}			/* unknown port */
@@ -244,7 +248,8 @@ static void proxy_init(server_rec *r, pool *p)
 /* The "ProxyDomain" directive determines what domain will be appended */
 static int proxy_needsdomain(request_rec *r, const char *url, const char *domain)
 {
-    char *nuri, *ref;
+    char *nuri;
+    const char *ref;
 
     /* We only want to worry about GETs */
     if (!r->proxyreq || r->method_number != M_GET || !r->parsed_uri.hostname)
@@ -266,7 +271,7 @@ static int proxy_needsdomain(request_rec *r, const char *url, const char *domain
 				  UNP_REVEALPASSWORD);
 
     ap_table_set(r->headers_out, "Location", nuri);
-    ap_log_error(APLOG_MARK, APLOG_INFO|APLOG_NOERRNO, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_INFO|APLOG_NOERRNO, r,
 		"Domain missing: %s sent to %s%s%s", r->uri,
 		ap_unparse_uri_components(r->pool, &r->parsed_uri,
 		      UNP_OMITUSERINFO),
@@ -287,12 +292,29 @@ static int proxy_handler(request_rec *r)
     array_header *proxies = conf->proxies;
     struct proxy_remote *ents = (struct proxy_remote *) proxies->elts;
     int i, rc;
-    struct cache_req *cr;
+    cache_req *cr;
     int direct_connect = 0;
+    const char *maxfwd_str;
 
     if (!r->proxyreq || strncmp(r->filename, "proxy:", 6) != 0)
 	return DECLINED;
 
+    if (r->method_number == M_TRACE &&
+	(maxfwd_str = ap_table_get(r->headers_in, "Max-Forwards")) != NULL) {
+	int maxfwd = strtol(maxfwd_str, NULL, 10);
+	if (maxfwd < 1) {
+	    int access_status;
+	    r->proxyreq = 0;
+	    if ((access_status = ap_send_http_trace(r)))
+		ap_die(access_status, r);
+	    else
+		ap_finalize_request_protocol(r);
+	    return OK;
+	}
+	ap_table_setn(r->headers_in, "Max-Forwards", 
+		      ap_psprintf(r->pool, "%d", (maxfwd > 0) ? maxfwd-1 : 0));
+    }
+
     if ((rc = ap_setup_client_block(r, REQUEST_CHUNKED_ERROR)))
 	return rc;
 
@@ -308,7 +330,7 @@ static int proxy_handler(request_rec *r)
     /* If the host doesn't have a domain name, add one and redirect. */
     if (conf->domain != NULL) {
 	rc = proxy_needsdomain(r, url, conf->domain);
-	if (is_HTTP_REDIRECT(rc))
+	if (ap_is_HTTP_REDIRECT(rc))
 	    return HTTP_MOVED_PERMANENTLY;
     }
 
@@ -328,7 +350,7 @@ static int proxy_handler(request_rec *r)
 	    direct_connect = list[ii].matcher(&list[ii], r);
 	}
 #if DEBUGGING
-	ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, r,
 		     (direct_connect) ? "NoProxy for %s" : "UseProxy for %s",
 		     r->uri);
 #endif
@@ -375,7 +397,7 @@ static int proxy_handler(request_rec *r)
     if (strcasecmp(scheme, "ftp") == 0)
 	return ap_proxy_ftp_handler(r, cr, url);
     else
-	return NOT_IMPLEMENTED;
+	return HTTP_FORBIDDEN;
 }
 
 /* -------------------------------------------------------------- */
@@ -392,7 +414,9 @@ static void *
     ps->noproxies = ap_make_array(p, 10, sizeof(struct noproxy_entry));
     ps->dirconn = ap_make_array(p, 10, sizeof(struct dirconn_entry));
     ps->nocaches = ap_make_array(p, 10, sizeof(struct nocache_entry));
+    ps->allowed_connect_ports = ap_make_array(p, 10, sizeof(int));
     ps->domain = NULL;
+    ps->viaopt = via_off; /* initially backward compatible with 1.3.1 */
     ps->req = 0;
 
     ps->cache.root = NULL;
@@ -404,6 +428,7 @@ static void *
     /* at these levels, the cache can have 2^18 directories (256,000)  */
     ps->cache.dirlevels = 3;
     ps->cache.dirlength = 1;
+    ps->cache.cache_completion = DEFAULT_CACHE_COMPLETION;
 
     return ps;
 }
@@ -510,6 +535,25 @@ static const char *
     return NULL;
 }
 
+/*
+ * Set the ports CONNECT can use
+ */
+static const char *
+    set_allowed_ports(cmd_parms *parms, void *dummy, char *arg)
+{
+    server_rec *s = parms->server;
+    proxy_server_conf *conf =
+      ap_get_module_config(s->module_config, &proxy_module);
+    int *New;
+
+    if (!ap_isdigit(arg[0]))
+	return "AllowCONNECT: port number must be numeric";
+
+    New = ap_push_array(conf->allowed_connect_ports);
+    *New = atoi(arg);
+    return NULL;
+}
+
 /* Similar to set_proxy_exclude(), but defining directly connected hosts,
  * which should never be accessed via the configured ProxyRemote servers
  */
@@ -741,6 +785,44 @@ static const char *
     return NULL;
 }
 
+static const char*
+    set_cache_completion(cmd_parms *parms, void *dummy, char *arg)
+{
+    proxy_server_conf *psf =
+    ap_get_module_config(parms->server->module_config, &proxy_module);
+    int s = atoi(arg);
+    if (s > 100 || s < 0) {
+	return "CacheForceCompletion must be <= 100 percent, "
+               "or 0 for system default.";
+    }
+
+    if (s > 0)
+      psf->cache.cache_completion = ((float)s / 100);
+    return NULL;    
+}
+
+static const char*
+    set_via_opt(cmd_parms *parms, void *dummy, char *arg)
+{
+    proxy_server_conf *psf =
+    ap_get_module_config(parms->server->module_config, &proxy_module);
+
+    if (strcasecmp(arg, "Off") == 0)
+        psf->viaopt = via_off;
+    else if (strcasecmp(arg, "On") == 0)
+        psf->viaopt = via_on;
+    else if (strcasecmp(arg, "Block") == 0)
+        psf->viaopt = via_block;
+    else if (strcasecmp(arg, "Full") == 0)
+        psf->viaopt = via_full;
+    else {
+	return "ProxyVia must be one of: "
+               "off | on | full | block";
+    }
+
+    return NULL;    
+}
+
 static const handler_rec proxy_handlers[] =
 {
     {"proxy-server", proxy_handler},
@@ -765,6 +847,8 @@ static const command_rec proxy_cmds[] =
      "A list of domains, hosts, or subnets to which the proxy will connect directly"},
     {"ProxyDomain", set_proxy_domain, NULL, RSRC_CONF, TAKE1,
      "The default intranet domain name (in absence of a domain in the URL)"},
+    {"AllowCONNECT", set_allowed_ports, NULL, RSRC_CONF, ITERATE,
+     "A list of ports which CONNECT may connect to"},
     {"CacheRoot", set_cache_root, NULL, RSRC_CONF, TAKE1,
      "The directory to store cache files"},
     {"CacheSize", set_cache_size, NULL, RSRC_CONF, TAKE1,
@@ -783,6 +867,10 @@ static const command_rec proxy_cmds[] =
      "The number of characters in subdirectory names"},
     {"NoCache", set_cache_exclude, NULL, RSRC_CONF, ITERATE,
      "A list of names, hosts or domains for which caching is *not* provided"},
+    {"CacheForceCompletion", set_cache_completion, NULL, RSRC_CONF, TAKE1,
+     "Force a http cache completion after this percentage is loaded"},
+    {"ProxyVia", set_via_opt, NULL, RSRC_CONF, TAKE1,
+     "Configure Via: proxy header header to one of: on | off | block | full"},
     {NULL}
 };
 
