@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1995-1998 The Apache Group.  All rights reserved.
+ * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -217,7 +217,8 @@ static const char *set_metafiles(cmd_parms *parms, cern_meta_dir_config * dconf,
 
 static const command_rec cern_meta_cmds[] =
 {
-    {"MetaFiles", set_metafiles, NULL, DIR_CMD_PERMS, FLAG, NULL},
+    {"MetaFiles", set_metafiles, NULL, DIR_CMD_PERMS, FLAG,
+    "Limited to 'on' or 'off'"},
     {"MetaDir", set_metadir, NULL, DIR_CMD_PERMS, TAKE1,
      "the name of the directory containing meta files"},
     {"MetaSuffix", set_metasuffix, NULL, DIR_CMD_PERMS, TAKE1,
@@ -225,12 +226,17 @@ static const command_rec cern_meta_cmds[] =
     {NULL}
 };
 
+/* XXX: this is very similar to ap_scan_script_header_err_core...
+ * are the differences deliberate, or just a result of bit rot?
+ */
 static int scan_meta_file(request_rec *r, FILE *f)
 {
     char w[MAX_STRING_LEN];
     char *l;
     int p;
+    table *tmp_headers;
 
+    tmp_headers = ap_make_table(r->pool, 5);
     while (fgets(w, MAX_STRING_LEN - 1, f) != NULL) {
 
 	/* Delete terminal (CR?)LF */
@@ -250,34 +256,36 @@ static int scan_meta_file(request_rec *r, FILE *f)
 	/* if we see a bogus header don't ignore it. Shout and scream */
 
 	if (!(l = strchr(w, ':'))) {
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
 			"malformed header in meta file: %s", r->filename);
 	    return SERVER_ERROR;
 	}
 
 	*l++ = '\0';
-	while (*l && isspace(*l))
+	while (*l && ap_isspace(*l))
 	    ++l;
 
 	if (!strcasecmp(w, "Content-type")) {
-
+	    char *tmp;
 	    /* Nuke trailing whitespace */
 
 	    char *endp = l + strlen(l) - 1;
-	    while (endp > l && isspace(*endp))
+	    while (endp > l && ap_isspace(*endp))
 		*endp-- = '\0';
 
-	    r->content_type = ap_pstrdup(r->pool, l);
-	    ap_str_tolower(r->content_type);
+	    tmp = ap_pstrdup(r->pool, l);
+	    ap_content_type_tolower(tmp);
+	    r->content_type = tmp;
 	}
 	else if (!strcasecmp(w, "Status")) {
 	    sscanf(l, "%d", &r->status);
 	    r->status_line = ap_pstrdup(r->pool, l);
 	}
 	else {
-	    ap_table_set(r->headers_out, w, l);
+	    ap_table_set(tmp_headers, w, l);
 	}
     }
+    ap_overlap_tables(r->headers_out, tmp_headers, AP_OVERLAP_TABLES_SET);
     return OK;
 }
 
@@ -322,7 +330,7 @@ static int add_cern_meta_data(request_rec *r)
     }
     else {
 	/* no last slash, buh?! */
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
 		    "internal error in mod_cern_meta: %s", r->filename);
 	/* should really barf, but hey, let's be friends... */
 	return DECLINED;
@@ -351,7 +359,7 @@ static int add_cern_meta_data(request_rec *r)
 	if (errno == ENOENT) {
 	    return DECLINED;
 	}
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 	      "meta file permissions deny server access: %s", metafilename);
 	return FORBIDDEN;
     };
