@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1995-1998 The Apache Group.  All rights reserved.
+ * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -628,7 +628,7 @@ static int include_cgi(char *s, request_rec *r)
  */
 static int is_only_below(const char *path)
 {
-#if WIN32
+#ifdef HAVE_DRIVE_LETTERS
     if (path[1] == ':')
 	return 0;
 #endif
@@ -688,13 +688,41 @@ static int handle_include(FILE *in, request_rec *r, const char *error, int noexe
                     "in parsed file %s";
             }
             if (error_fmt == NULL) {
+		/* try to avoid recursive includes.  We do this by walking
+		 * up the r->main list of subrequests, and at each level
+		 * walking back through any internal redirects.  At each
+		 * step, we compare the filenames and the URIs.  
+		 *
+		 * The filename comparison catches a recursive include
+		 * with an ever-changing URL, eg.
+		 * <!--#include virtual=
+		 *      "$REQUEST_URI/$QUERY_STRING?$QUERY_STRING/x"-->
+		 * which, although they would eventually be caught because
+		 * we have a limit on the length of files, etc., can 
+		 * recurse for a while.
+		 *
+		 * The URI comparison catches the case where the filename
+		 * is changed while processing the request, so the 
+		 * current name is never the same as any previous one.
+		 * This can happen with "DocumentRoot /foo" when you
+		 * request "/" on the server and it includes "/".
+		 * This only applies to modules such as mod_dir that 
+		 * (somewhat improperly) mess with r->filename outside 
+		 * of a filename translation phase.
+		 */
+		int founddupe = 0;
                 request_rec *p;
+                for (p = r; p != NULL && !founddupe; p = p->main) {
+		    request_rec *q;
+		    for (q = p; q != NULL; q = q->prev) {
+			if ( (strcmp(q->filename, rr->filename) == 0) ||
+			     (strcmp(q->uri, rr->uri) == 0) ){
+			    founddupe = 1;
+			    break;
+			}
+		    }
+		}
 
-                for (p = r; p != NULL; p = p->main) {
-                    if (strcmp(p->filename, rr->filename) == 0) {
-                        break;
-                    }
-                }
                 if (p != NULL) {
                     error_fmt = "Recursive include of \"%s\" "
                         "in parsed file %s";
@@ -737,6 +765,9 @@ static int handle_include(FILE *in, request_rec *r, const char *error, int noexe
 }
 
 typedef struct {
+#ifdef TPF
+    TPF_FORK_CHILD t;
+#endif
     request_rec *r;
     char *s;
 } include_cmd_arg;
@@ -768,7 +799,7 @@ static int include_cmd_child(void *arg, child_info *pinfo)
 
         ap_table_setn(env, "PATH_INFO", ap_escape_shell_cmd(r->pool, r->path_info));
 
-        pa_req = ap_sub_req_lookup_uri(escape_uri(r->pool, r->path_info), r);
+        pa_req = ap_sub_req_lookup_uri(ap_escape_uri(r->pool, r->path_info), r);
         if (pa_req->filename) {
             ap_table_setn(env, "PATH_TRANSLATED",
                       ap_pstrcat(r->pool, pa_req->filename, pa_req->path_info,
@@ -790,11 +821,14 @@ static int include_cmd_child(void *arg, child_info *pinfo)
 #ifdef DEBUG_INCLUDE_CMD
     fprintf(dbg, "Attempting to exec '%s'\n", s);
 #endif
+#ifdef TPF
+    return (0);
+#else
     ap_cleanup_for_exec();
     /* set shellcmd flag to pass arg to SHELL_PATH */
     child_pid = ap_call_exec(r, pinfo, s, ap_create_environment(r->pool, env),
 			     1);
-#ifdef WIN32
+#if defined(WIN32) || defined(OS2)
     return (child_pid);
 #else
     /* Oh, drat.  We're still here.  The log file descriptors are closed,
@@ -805,13 +839,14 @@ static int include_cmd_child(void *arg, child_info *pinfo)
     fprintf(dbg, "Exec failed\n");
 #endif
     ap_snprintf(err_string, sizeof(err_string),
-                "httpd: exec of %s failed, reason: %s (errno = %d)\n",
+                "exec of %s failed, reason: %s (errno = %d)\n",
                 SHELL_PATH, strerror(errno), errno);
     write(STDERR_FILENO, err_string, strlen(err_string));
     exit(0);
     /* NOT REACHED */
     return (child_pid);
 #endif /* WIN32 */
+#endif /* TPF */
 }
 
 static int include_cmd(char *s, request_rec *r)
@@ -821,6 +856,11 @@ static int include_cmd(char *s, request_rec *r)
 
     arg.r = r;
     arg.s = s;
+#ifdef TPF
+    arg.t.filename = r->filename;
+    arg.t.subprocess_env = r->subprocess_env;
+    arg.t.prog_type = FORK_FILE;
+#endif
 
     if (!ap_bspawn_child(r->pool, include_cmd_child, &arg,
 			 kill_after_timeout, NULL, &script_in, NULL)) {
@@ -924,10 +964,10 @@ static int handle_perl(FILE *in, request_rec *r, const char *error)
     SV *sub = Nullsv;
     AV *av = newAV();
 
-    if (!(ap_allow_options(r) & OPT_INCLUDES)) {
+    if (ap_allow_options(r) & OPT_INCNOEXEC) {
         ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
-                    "httpd: #perl SSI disallowed by IncludesNoExec in %s",
-                    r->filename);
+		      "#perl SSI disallowed by IncludesNoExec in %s",
+		      r->filename);
         return DECLINED;
     }
     while (1) {
@@ -1005,35 +1045,41 @@ static int handle_config(FILE *in, request_rec *r, char *error, char *tf,
 static int find_file(request_rec *r, const char *directive, const char *tag,
                      char *tag_val, struct stat *finfo, const char *error)
 {
-    char *to_send;
-    request_rec *rr;
+    char *to_send = tag_val;
+    request_rec *rr = NULL;
     int ret=0;
+    char *error_fmt = NULL;
 
     if (!strcmp(tag, "file")) {
-        ap_getparents(tag_val);    /* get rid of any nasties */
-        
-        rr = ap_sub_req_lookup_file(tag_val, r);
-
-        if (rr->status == HTTP_OK && rr->finfo.st_mode != 0) {
-            to_send = rr->filename;
-            if ((ret = stat(to_send, finfo)) == -1) {
-                ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-                            "unable to get information about \"%s\" "
-                            "in parsed file %s",
-                            to_send, r->filename);
-                ap_rputs(error, r);
-            }
+        /* be safe; only files in this directory or below allowed */
+        if (!is_only_below(tag_val)) {
+            error_fmt = "unable to access file \"%s\" "
+                        "in parsed file %s";
         }
         else {
+            ap_getparents(tag_val);    /* get rid of any nasties */
+            rr = ap_sub_req_lookup_file(tag_val, r);
+
+            if (rr->status == HTTP_OK && rr->finfo.st_mode != 0) {
+                to_send = rr->filename;
+                if (stat(to_send, finfo)) {
+                    error_fmt = "unable to get information about \"%s\" "
+                                "in parsed file %s";
+                }
+            }
+            else {
+                error_fmt = "unable to lookup information about \"%s\" "
+                            "in parsed file %s";
+            }
+        }
+
+        if (error_fmt) {
             ret = -1;
-            ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-                        "unable to lookup information about \"%s\" "
-                        "in parsed file %s",
-                        tag_val, r->filename);
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, r, error_fmt, to_send, r->filename);
             ap_rputs(error, r);
         }
-        
-        ap_destroy_sub_req(rr);
+
+        if (rr) ap_destroy_sub_req(rr);
         
         return ret;
     }
@@ -1088,9 +1134,8 @@ static int handle_fsize(FILE *in, request_rec *r, const char *error, int sizefmt
                 }
                 else {
                     int l, x;
-#if defined(BSD) && BSD > 199305
-                    /* ap_snprintf can't handle %qd */
-                    sprintf(tag, "%qd", finfo.st_size);
+#if defined(AP_OFF_T_IS_QUAD)
+                    ap_snprintf(tag, sizeof(tag), "%qd", finfo.st_size);
 #else
                     ap_snprintf(tag, sizeof(tag), "%ld", finfo.st_size);
 #endif
@@ -1141,7 +1186,7 @@ static int re_check(request_rec *r, char *string, char *rexp)
                     "unable to compile pattern \"%s\"", rexp);
         return -1;
     }
-    regex_error = regexec(compiled, string, 0, (regmatch_t *) NULL, 0);
+    regex_error = ap_regexec(compiled, string, 0, (regmatch_t *) NULL, 0);
     ap_pregfree(r->pool, compiled);
     return (!regex_error);
 }
@@ -2176,8 +2221,8 @@ static void send_parsed_content(FILE *f, request_rec *r)
             if (!strcmp(directive, "exec")) {
                 if (noexec) {
                     ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
-                                "httpd: exec used but not allowed in %s",
-                                r->filename);
+				  "exec used but not allowed in %s",
+				  r->filename);
                     if (printing) {
                         ap_rputs(error, r);
                     }
@@ -2215,9 +2260,9 @@ static void send_parsed_content(FILE *f, request_rec *r)
 #endif
             else {
                 ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
-                            "httpd: unknown directive \"%s\" "
-                            "in parsed doc %s",
-                            directive, r->filename);
+			      "unknown directive \"%s\" "
+			      "in parsed doc %s",
+			      directive, r->filename);
                 if (printing) {
                     ap_rputs(error, r);
                 }
@@ -2225,8 +2270,8 @@ static void send_parsed_content(FILE *f, request_rec *r)
             }
             if (ret) {
                 ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
-                            "httpd: premature EOF in parsed file %s",
-                            r->filename);
+			      "premature EOF in parsed file %s",
+			      r->filename);
                 return;
             }
         }
