@@ -122,6 +122,7 @@
  * %...f:  filename
  * %...h:  remote host
  * %...a:  remote IP-address
+ * %...A:  local IP-address
  * %...{Foobar}i:  The contents of Foobar: header line(s) in the request
  *                 sent to the client.
  * %...l:  remote logname (from identd, if supplied)
@@ -289,6 +290,11 @@ static const char *log_remote_address(request_rec *r, char *a)
     return r->connection->remote_ip;
 }
 
+static const char *log_local_address(request_rec *r, char *a)
+{
+    return r->connection->local_ip;
+}
+
 static const char *log_remote_logname(request_rec *r, char *a)
 {
     return ap_get_remote_logname(r);
@@ -384,16 +390,14 @@ static const char *log_request_time(request_rec *r, char *a)
     }
     else {                      /* CLF format */
         char sign = (timz < 0 ? '-' : '+');
-	size_t l;
 
         if (timz < 0) {
             timz = -timz;
         }
-
-        strftime(tstr, MAX_STRING_LEN, "[%d/%b/%Y:%H:%M:%S ", t);
-	l = strlen(tstr);
-        ap_snprintf(tstr + l, sizeof(tstr) - l,
-                    "%c%.2d%.2d]", sign, timz / 60, timz % 60);
+        ap_snprintf(tstr, sizeof(tstr), "[%02d/%s/%d:%02d:%02d:%02d %c%.2d%.2d]",
+                t->tm_mday, ap_month_snames[t->tm_mon], t->tm_year+1900, 
+                t->tm_hour, t->tm_min, t->tm_sec,
+                sign, timz / 60, timz % 60);
     }
 
     return ap_pstrdup(r->pool, tstr);
@@ -448,6 +452,9 @@ static struct log_item_list {
     {   
         'a', log_remote_address, 0 
     },
+    {   
+        'A', log_local_address, 0 
+    },
     {
         'l', log_remote_logname, 0
     },
