@@ -1,6 +1,6 @@
 #define WANT_BASENAME_MATCH
 /* ====================================================================
- * Copyright (c) 1996-1998 The Apache Group.  All rights reserved.
+ * Copyright (c) 1996-1999 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -68,7 +68,8 @@
  * capitalizations. If it finds a match, it sends a redirect.
  *
  * 08-Aug-1997 <Martin.Kraemer@Mch.SNI.De>
- * o Upgraded module interface to apache_1.3a2-dev API (more NULL's in speling_module).
+ * o Upgraded module interface to apache_1.3a2-dev API (more NULL's in
+ *   speling_module).
  * o Integrated tcsh's "spelling correction" routine which allows one
  *   misspelling (character insertion/omission/typo/transposition).
  *   Rewrote it to ignore case as well. This ought to catch the majority
@@ -84,29 +85,64 @@
 
 MODULE_VAR_EXPORT module speling_module;
 
-/* We use the "unconventional" mod_userdir approach here. And heck,
- * here it's just one int!
+typedef struct {
+    int enabled;
+} spconfig;
+
+/*
+ * Create a configuration specific to this module for a server or directory
+ * location, and fill it with the default settings.
+ *
+ * The API says that in the absence of a merge function, the record for the
+ * closest ancestor is used exclusively.  That's what we want, so we don't
+ * bother to have such a function.
  */
 
-static void *create_speling_config(pool *dummy, server_rec *s)
+static void *mkconfig(pool *p)
 {
-    return (void *) 0;
+    spconfig *cfg = ap_pcalloc(p, sizeof(spconfig));
+
+    cfg->enabled = 0;
+    return cfg;
 }
 
-static const char *set_speling(cmd_parms *cmd, void *dummy, int arg)
+/*
+ * Respond to a callback to create configuration record for a server or
+ * vhost environment.
+ */
+static void *create_mconfig_for_server(pool *p, server_rec *s)
 {
-    void *server_conf = cmd->server->module_config;
+    return mkconfig(p);
+}
 
-    /* any non-NULL pointer means speling is enabled */
-    ap_set_module_config(server_conf, &speling_module, arg ? (void *) &speling_module : NULL);
+/*
+ * Respond to a callback to create a config record for a specific directory.
+ */
+static void *create_mconfig_for_directory(pool *p, char *dir)
+{
+    return mkconfig(p);
+}
+
+/*
+ * Handler for the CheckSpelling directive, which is FLAG.
+ */
+static const char *set_speling(cmd_parms *cmd, void *mconfig, int arg)
+{
+    spconfig *cfg = (spconfig *) mconfig;
+
+    cfg->enabled = arg;
     return NULL;
 }
 
+/*
+ * Define the directives specific to this module.  This structure is referenced
+ * later by the 'module' structure.
+ */
 static const command_rec speling_cmds[] =
 {
-    {"CheckSpelling", set_speling, NULL, RSRC_CONF, FLAG,
-     "whether or not to fix miscapitalized/misspelled requests"},
-    {NULL}
+    { "CheckSpelling", set_speling, NULL, OR_OPTIONS, FLAG,
+      "whether or not to fix miscapitalized/misspelled requests" },
+    { NULL }
 };
 
 typedef enum {
@@ -156,22 +192,29 @@ typedef struct {
 
 static sp_reason spdist(const char *s, const char *t)
 {
-    for (; tolower(*s) == tolower(*t); t++, s++)
-        if (*t == '\0')
+    for (; ap_tolower(*s) == ap_tolower(*t); t++, s++) {
+        if (*t == '\0') {
             return SP_MISCAPITALIZED;   /* exact match (sans case) */
+	}
+    }
     if (*s) {
         if (*t) {
-            if (s[1] && t[1] && tolower(*s) == tolower(t[1]) &&
-              tolower(*t) == tolower(s[1]) && strcasecmp(s + 2, t + 2) == 0)
+            if (s[1] && t[1] && ap_tolower(*s) == ap_tolower(t[1])
+		&& ap_tolower(*t) == ap_tolower(s[1])
+		&& strcasecmp(s + 2, t + 2) == 0) {
                 return SP_TRANSPOSITION;        /* transposition */
-            if (strcasecmp(s + 1, t + 1) == 0)
+	    }
+            if (strcasecmp(s + 1, t + 1) == 0) {
                 return SP_SIMPLETYPO;   /* 1 char mismatch */
+	    }
         }
-        if (strcasecmp(s + 1, t) == 0)
+        if (strcasecmp(s + 1, t) == 0) {
             return SP_EXTRACHAR;        /* extra character */
+	}
     }
-    if (*t && strcasecmp(s, t + 1) == 0)
+    if (*t && strcasecmp(s, t + 1) == 0) {
         return SP_MISSINGCHAR;  /* missing character */
+    }
     return SP_VERYDIFFERENT;    /* distance too large to fix. */
 }
 
@@ -183,27 +226,32 @@ static int sort_by_quality(const void *left, const void *rite)
 
 static int check_speling(request_rec *r)
 {
-    void *server_conf = r->server->module_config;
+    spconfig *cfg;
     char *good, *bad, *postgood, *url;
     int filoc, dotloc, urlen, pglen;
     DIR *dirp;
     struct DIR_TYPE *dir_entry;
     array_header *candidates = NULL;
 
-    if (!ap_get_module_config(server_conf, &speling_module))
+    cfg = ap_get_module_config(r->per_dir_config, &speling_module);
+    if (!cfg->enabled) {
         return DECLINED;
+    }
 
     /* We only want to worry about GETs */
-    if (r->method_number != M_GET)
+    if (r->method_number != M_GET) {
         return DECLINED;
+    }
 
     /* We've already got a file of some kind or another */
-    if (r->proxyreq || (r->finfo.st_mode != 0))
+    if (r->proxyreq || (r->finfo.st_mode != 0)) {
         return DECLINED;
+    }
 
     /* This is a sub request - don't mess with it */
-    if (r->main)
+    if (r->main) {
         return DECLINED;
+    }
 
     /*
      * The request should end up looking like this:
@@ -214,9 +262,13 @@ static int check_speling(request_rec *r)
      */
 
     filoc = ap_rind(r->filename, '/');
-    /* Don't do anything if the request doesn't contain a slash, or requests "/" */
-    if (filoc == -1 || strcmp(r->uri, "/") == 0)
+    /*
+     * Don't do anything if the request doesn't contain a slash, or
+     * requests "/" 
+     */
+    if (filoc == -1 || strcmp(r->uri, "/") == 0) {
         return DECLINED;
+    }
 
     /* good = /correct-file */
     good = ap_pstrndup(r->pool, r->filename, filoc);
@@ -229,22 +281,25 @@ static int check_speling(request_rec *r)
     pglen = strlen(postgood);
 
     /* Check to see if the URL pieces add up */
-    if (strcmp(postgood, r->uri + (urlen - pglen)))
+    if (strcmp(postgood, r->uri + (urlen - pglen))) {
         return DECLINED;
+    }
 
     /* url = /correct-url */
     url = ap_pstrndup(r->pool, r->uri, (urlen - pglen));
 
     /* Now open the directory and do ourselves a check... */
     dirp = ap_popendir(r->pool, good);
-    if (dirp == NULL)           /* Oops, not a directory... */
+    if (dirp == NULL) {          /* Oops, not a directory... */
         return DECLINED;
+    }
 
     candidates = ap_make_array(r->pool, 2, sizeof(misspelled_file));
 
     dotloc = ap_ind(bad, '.');
-    if (dotloc == -1)
+    if (dotloc == -1) {
         dotloc = strlen(bad);
+    }
 
     while ((dir_entry = readdir(dirp)) != NULL) {
         sp_reason q;
@@ -263,7 +318,9 @@ static int check_speling(request_rec *r)
          * file, upper case request)
          */
         else if (strcasecmp(bad, dir_entry->d_name) == 0) {
-            misspelled_file *sp_new = (misspelled_file *) ap_push_array(candidates);
+            misspelled_file *sp_new;
+
+	    sp_new = (misspelled_file *) ap_push_array(candidates);
             sp_new->name = ap_pstrdup(r->pool, dir_entry->d_name);
             sp_new->quality = SP_MISCAPITALIZED;
         }
@@ -272,26 +329,31 @@ static int check_speling(request_rec *r)
          * missing/extra/transposed char)
          */
         else if ((q = spdist(bad, dir_entry->d_name)) != SP_VERYDIFFERENT) {
-            misspelled_file *sp_new = (misspelled_file *) ap_push_array(candidates);
+            misspelled_file *sp_new;
+
+	    sp_new = (misspelled_file *) ap_push_array(candidates);
             sp_new->name = ap_pstrdup(r->pool, dir_entry->d_name);
             sp_new->quality = q;
         }
-        /* The spdist() should have found the majority of the misspelled requests.
-         * it is of questionable use to continue looking for files with the same
-         * base name, but potentially of totally wrong type (index.html <-> index.db)
-         * I would propose to not set the WANT_BASENAME_MATCH define.
+        /*
+	 * The spdist() should have found the majority of the misspelled
+	 * requests.  It is of questionable use to continue looking for
+	 * files with the same base name, but potentially of totally wrong
+	 * type (index.html <-> index.db).
+	 * I would propose to not set the WANT_BASENAME_MATCH define.
          *      08-Aug-1997 <Martin.Kraemer@Mch.SNI.De>
          *
          * However, Alexei replied giving some reasons to add it anyway:
          * > Oh, by the way, I remembered why having the
          * > extension-stripping-and-matching stuff is a good idea:
          * >
-         * > If you're using MultiViews, and have a file named foobar.html, which you
-         * > refer to as "foobar", and someone tried to access "Foobar", mod_speling
-         * > won't find it, because it won't find anything matching that
-         * > spelling. With the extension-munging, it would locate "foobar.html". Not
-         * > perfect, but I ran into that problem when I first wrote the module.
-         */
+         * > If you're using MultiViews, and have a file named foobar.html,
+	 * > which you refer to as "foobar", and someone tried to access
+	 * > "Foobar", mod_speling won't find it, because it won't find
+	 * > anything matching that spelling. With the extension-munging,
+	 * > it would locate "foobar.html". Not perfect, but I ran into
+	 * > that problem when I first wrote the module.
+	 */
         else {
 #ifdef WANT_BASENAME_MATCH
             /*
@@ -306,12 +368,15 @@ static int check_speling(request_rec *r)
              * it finds. Better than a Not Found, though.
              */
             int entloc = ap_ind(dir_entry->d_name, '.');
-            if (entloc == -1)
+            if (entloc == -1) {
                 entloc = strlen(dir_entry->d_name);
+	    }
 
             if ((dotloc == entloc)
                 && !strncasecmp(bad, dir_entry->d_name, dotloc)) {
-                misspelled_file *sp_new = (misspelled_file *) ap_push_array(candidates);
+                misspelled_file *sp_new;
+
+		sp_new = (misspelled_file *) ap_push_array(candidates);
                 sp_new->name = ap_pstrdup(r->pool, dir_entry->d_name);
                 sp_new->quality = SP_VERYDIFFERENT;
             }
@@ -322,7 +387,8 @@ static int check_speling(request_rec *r)
 
     if (candidates->nelts != 0) {
         /* Wow... we found us a mispelling. Construct a fixed url */
-        char *nuri, *ref;
+        char *nuri;
+	const char *ref;
         misspelled_file *variant = (misspelled_file *) candidates->elts;
         int i;
 
@@ -334,23 +400,26 @@ static int check_speling(request_rec *r)
         /*
          * Conditions for immediate redirection: 
          *     a) the first candidate was not found by stripping the suffix 
-         * AND b) there exists only one candidate OR the best match is not ambigous
+         * AND b) there exists only one candidate OR the best match is not
+	 *        ambiguous
          * then return a redirection right away.
          */
-        if (variant[0].quality != SP_VERYDIFFERENT &&
-            (candidates->nelts == 1 || variant[0].quality != variant[1].quality)) {
+        if (variant[0].quality != SP_VERYDIFFERENT
+	    && (candidates->nelts == 1
+		|| variant[0].quality != variant[1].quality)) {
 
             nuri = ap_pstrcat(r->pool, url, variant[0].name, r->path_info,
 			      r->parsed_uri.query ? "?" : "",
-			      r->parsed_uri.query ? r->parsed_uri.query : "", NULL);
+			      r->parsed_uri.query ? r->parsed_uri.query : "",
+			      NULL);
 
             ap_table_setn(r->headers_out, "Location",
-                      ap_construct_url(r->pool, nuri, r));
+			  ap_construct_url(r->pool, nuri, r));
 
-            ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_INFO, r->server,
-                        ref ? "Fixed spelling: %s to %s from %s"
-                        : "Fixed spelling: %s to %s",
-                        r->uri, nuri, ref);
+            ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_INFO, r,
+			 ref ? "Fixed spelling: %s to %s from %s"
+			     : "Fixed spelling: %s to %s",
+			 r->uri, nuri, ref);
 
             return HTTP_MOVED_PERMANENTLY;
         }
@@ -359,9 +428,12 @@ static int check_speling(request_rec *r)
          * returned.
          */
         else {
-            char *t;
             pool *p;
             table *notes;
+	    pool *sub_pool;
+	    array_header *t;
+	    array_header *v;
+
 
             if (r->main == NULL) {
                 p = r->pool;
@@ -372,26 +444,47 @@ static int check_speling(request_rec *r)
                 notes = r->main->notes;
             }
 
+	    sub_pool = ap_make_sub_pool(p);
+	    t = ap_make_array(sub_pool, candidates->nelts * 8 + 8,
+			      sizeof(char *));
+	    v = ap_make_array(sub_pool, candidates->nelts * 5,
+			      sizeof(char *));
+
             /* Generate the response text. */
-            /* Since the text is expanded by repeated calls of
-             * t = pstrcat(p, t, ".."), we can avoid a little waste
-             * of memory by adding the header AFTER building the list.
-             * XXX: FIXME: find a way to build a string concatenation
-             *             without repeatedly requesting new memory
-             * XXX: FIXME: Limit the list to a maximum number of entries
-             */
-            t = "";
+
+	    *(const char **)ap_push_array(t) =
+			  "The document name you requested (<code>";
+	    *(const char **)ap_push_array(t) = r->uri;
+	    *(const char **)ap_push_array(t) =
+			   "</code>) could not be found on this server.\n"
+			   "However, we found documents with names similar "
+			   "to the one you requested.<p>"
+			   "Available documents:\n<ul>\n";
 
             for (i = 0; i < candidates->nelts; ++i) {
+		char *vuri;
+		const char *reason;
 
+		reason = sp_reason_str[(int) (variant[i].quality)];
                 /* The format isn't very neat... */
-                t = ap_pstrcat(p, t, "<li><a href=\"", url,
-			       variant[i].name, r->path_info, "\">",
-			       variant[i].name, r->path_info,
-			       r->parsed_uri.query ? "?" : "",
-			       r->parsed_uri.query ? r->parsed_uri.query : "",
-			       "</a> (",
-                    sp_reason_str[(int) (variant[i].quality)], ")\n", NULL);
+		vuri = ap_pstrcat(sub_pool, url, variant[i].name, r->path_info,
+				  (r->parsed_uri.query != NULL) ? "?" : "",
+				  (r->parsed_uri.query != NULL)
+				      ? r->parsed_uri.query : "",
+				  NULL);
+		*(const char **)ap_push_array(v) = "\"";
+		*(const char **)ap_push_array(v) = vuri;
+		*(const char **)ap_push_array(v) = "\";\"";
+		*(const char **)ap_push_array(v) = reason;
+		*(const char **)ap_push_array(v) = "\"";
+
+		*(const char **)ap_push_array(t) = "<li><a href=\"";
+		*(const char **)ap_push_array(t) = vuri;
+		*(const char **)ap_push_array(t) = "\">";
+		*(const char **)ap_push_array(t) = vuri;
+		*(const char **)ap_push_array(t) = "</a> (";
+		*(const char **)ap_push_array(t) = reason;
+		*(const char **)ap_push_array(t) = ")\n";
 
                 /*
                  * when we have printed the "close matches" and there are
@@ -403,26 +496,36 @@ static int check_speling(request_rec *r)
                 if (i > 0 && i < candidates->nelts - 1
                     && variant[i].quality != SP_VERYDIFFERENT
                     && variant[i + 1].quality == SP_VERYDIFFERENT) {
-                    t = ap_pstrcat(p, t, "</ul>\nFurthermore, the following related documents were found:\n<ul>\n", NULL);
+		    *(const char **)ap_push_array(t) = 
+				   "</ul>\nFurthermore, the following related "
+				   "documents were found:\n<ul>\n";
                 }
             }
-            t = ap_pstrcat(p, "The document name you requested (<code>",
-                     r->uri, "</code>) could not be found on this server.\n"
-                        "However, we found documents with names similar to the one you requested.<p>"
-                        "Available documents:\n<ul>\n", t, "</ul>\n", NULL);
+	    *(const char **)ap_push_array(t) = "</ul>\n";
 
             /* If we know there was a referring page, add a note: */
-            if (ref != NULL)
-                t = ap_pstrcat(p, t, "Please consider informing the owner of the <a href=\"",
-                ref, "\">referring page</a> about the broken link.\n", NULL);
+            if (ref != NULL) {
+                *(const char **)ap_push_array(t) =
+			       "Please consider informing the owner of the "
+			       "<a href=\"";
+                *(const char **)ap_push_array(t) = ref;
+                *(const char **)ap_push_array(t) = "\">referring page</a> "
+			       "about the broken link.\n";
+	    }
+
 
             /* Pass our table to http_protocol.c (see mod_negotiation): */
-            ap_table_setn(notes, "variant-list", t);
+            ap_table_setn(notes, "variant-list", ap_array_pstrcat(p, t, 0));
 
-            ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_INFO, r->server,
-                        ref ? "Spelling fix: %s: %d candidates from %s"
-                        : "Spelling fix: %s: %d candidates",
-                        r->uri, candidates->nelts, ref);
+	    ap_table_mergen(r->subprocess_env, "VARIANTS",
+			    ap_array_pstrcat(p, v, ','));
+	  
+	    ap_destroy_pool(sub_pool);
+
+            ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_INFO, r,
+			 ref ? "Spelling fix: %s: %d candidates from %s"
+			     : "Spelling fix: %s: %d candidates",
+			 r->uri, candidates->nelts, ref);
 
             return HTTP_MULTIPLE_CHOICES;
         }
@@ -435,9 +538,9 @@ module MODULE_VAR_EXPORT speling_module =
 {
     STANDARD_MODULE_STUFF,
     NULL,                       /* initializer */
-    NULL,                       /* create per-dir config */
+    create_mconfig_for_directory,  /* create per-dir config */
     NULL,                       /* merge per-dir config */
-    create_speling_config,      /* server config */
+    create_mconfig_for_server,  /* server config */
     NULL,                       /* merge server config */
     speling_cmds,               /* command table */
     NULL,                       /* handlers */
