@@ -15,6 +15,16 @@
  * release to a development or beta version.
  */
 
+/* To allow for multiple services, store the configuration file's full path
+ * under each service entry:
+ *
+ * HKLM\System\CurrentControlSet\Services\[service name]\Parameters\ConfPath
+ *
+ * The default configuration path (for console apache) is still stored:
+ * 
+ * HKLM\Software\[Vendor]\[Software]\[Version]\ServerRoot
+ */
+
 #include <windows.h>
 #include <stdio.h>
 
@@ -28,10 +38,13 @@
 
 #define VENDOR   "Apache Group"
 #define SOFTWARE "Apache"
-#define VERSION  "1.3.6"
+#define VERSION  "1.3.9"
 
 #define REGKEY "SOFTWARE\\" VENDOR "\\" SOFTWARE "\\" VERSION
 
+#define SERVICEKEYPRE  "System\\CurrentControlSet\\Services\\"
+#define SERVICEKEYPOST "\\Parameters"
+
 /*
  * The Windows API registry key functions don't set the last error
  * value (the windows equivalent of errno). So we need to set it
@@ -79,7 +92,7 @@
  * message will be logged at priority "warning".
  */
 
-static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeBuffer, char **ppValue)
+static int ap_registry_get_key_int(pool *p, char *key, char *name, char *pBuffer, int nSizeBuffer, char **ppValue)
 {
     long rv;
     HKEY hKey;
@@ -88,19 +101,18 @@ static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeB
     int retval;
 
     rv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		      REGKEY,
+                      key,
 		      0,
 		      KEY_READ,
 		      &hKey);
 
     if (rv == ERROR_FILE_NOT_FOUND) {
 	ap_log_error(APLOG_MARK,APLOG_WARNING|APLOG_NOERRNO,NULL,
-	    "Registry does not contain key " REGKEY);
+        "Registry does not contain key %s",key);
 	return -1;
     }
     if (rv != ERROR_SUCCESS) {
-	do_error(rv, "RegOpenKeyEx HKLM\\" REGKEY,
-		 NULL);
+        do_error(rv, "RegOpenKeyEx HKLM\\%s",key);
 	return -4;
     }
 
@@ -110,7 +122,7 @@ static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeB
 	 * buffer if the return value is ERROR_SUCCESS.
 	 */
 	rv = RegQueryValueEx(hKey, 
-			     key,		/* key name */
+			     name,		/* key name */
 			     NULL,		/* reserved */
 			     NULL,		/* type */
 			     NULL,		/* for value */
@@ -139,7 +151,7 @@ static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeB
     }
 
     rv = RegQueryValueEx(hKey, 
-			 key,		/* key name */
+			 name,		/* key name */
 			 NULL,		/* reserved */
 			 NULL,		/* type */
 			 pValue,		/* for value */
@@ -149,7 +161,7 @@ static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeB
 
     if (rv == ERROR_FILE_NOT_FOUND) {
 	ap_log_error(APLOG_MARK,APLOG_WARNING|APLOG_NOERRNO,NULL,
-	    "Registry does not contain value " REGKEY "\\%s", key);
+        "Registry does not contain value %s\\%s", key, name);
 	retval = -1;
     }
     else if (rv == ERROR_MORE_DATA) {
@@ -169,7 +181,7 @@ static int ap_registry_get_key_int(pool *p, char *key, char *pBuffer, int nSizeB
 
     rv = RegCloseKey(hKey);
     if (rv != ERROR_SUCCESS) {
-	do_error(rv, "RegCloseKey HKLM\\" REGKEY, NULL);
+    do_error(rv, "RegCloseKey HKLM\\%s", key);
 	if (retval == 0) {
 	    /* Keep error status from RegQueryValueEx, if any */
 	    retval = -4;  
@@ -191,7 +203,7 @@ int ap_registry_get_server_root(pool *p, char *dir, int size)
 {
     int rv;
 
-    rv = ap_registry_get_key_int(p, "ServerRoot", dir, size, NULL);
+    rv = ap_registry_get_key_int(p, REGKEY, "ServerRoot", dir, size, NULL);
     if (rv < 0) {
 	dir[0] = '\0';
     }
@@ -199,10 +211,61 @@ int ap_registry_get_server_root(pool *p, char *dir, int size)
     return (rv < -1) ? -1 : 0;
 }
 
+char *ap_get_service_key(char *service_name)
+{
+    char *key = malloc(strlen(SERVICEKEYPRE) +
+                       strlen(service_name) +
+                       strlen(SERVICEKEYPOST) + 1);
+
+    sprintf(key,"%s%s%s", SERVICEKEYPRE, service_name, SERVICEKEYPOST);
+
+    return(key);
+}
+
+int ap_registry_get_service_conf(pool *p, char *dir, int size, char *service_name)
+{
+    int rv;
+    char *key = ap_get_service_key(service_name);
+
+    rv = ap_registry_get_key_int(p, key, "ConfPath", dir, size, NULL);
+    if (rv < 0) {
+    dir[0] = '\0';
+    }
+
+    free(key);
+    return (rv < -1) ? -1 : 0;
+}
+
 /**********************************************************************
  * The rest of this file deals with storing keys or values in the registry
  */
 
+char *ap_registry_parse_key(int index, char *key)
+{
+    char *head = key, *skey;
+    int i;
+    
+    if(!key)
+        return(NULL);
+
+    for(i = 0; i <= index; i++)
+    {
+        if(key && key[0] == '\\')
+            key++;
+        if (!key)
+            return(NULL);
+        head = key;
+        key = strchr(head, '\\');
+    }
+
+    if(!key)
+        return(strdup(head));
+    *key = '\0';
+    skey = strdup(head);
+    *key = '\\';
+    return(skey);
+}
+
 /*
  * ap_registry_create_apache_key() creates the Apache registry key
  * (HLKM\SOFTWARE\Apache Group\Apache\version, as defined at the start
@@ -215,31 +278,25 @@ int ap_registry_get_server_root(pool *p, char *dir, int size)
  * already have been logged.
  */
 
-static int ap_registry_create_apache_key(void)
+static int ap_registry_create_key(char *longkey)
 {
-    static char *keys[] = 
-    { "SOFTWARE",
-	VENDOR,
-	SOFTWARE,
-	VERSION,
-	NULL
-    };
     int index;
     HKEY hKey;
     HKEY hKeyNext;
     int retval;
     int rv;
+    char *key;
 
     hKey = HKEY_LOCAL_MACHINE;
     index = 0;
     retval = 0;
 
     /* Walk the tree, creating at each stage if necessary */
-    while (keys[index]) {
+    while (key=ap_registry_parse_key(index,longkey)) {
 	int result;
 
 	rv = RegCreateKeyEx(hKey,
-			    keys[index], /* subkey */
+			    key,         /* subkey */
 			    0,	         /* reserved */
 			    NULL,        /* class */
 			    REG_OPTION_NON_VOLATILE,
@@ -248,7 +305,7 @@ static int ap_registry_create_apache_key(void)
 			    &hKeyNext,
 			    &result);
 	if (rv != ERROR_SUCCESS) {
-	    do_error(rv, "RegCreateKeyEx(%s)", keys[index]);
+	    do_error(rv, "RegCreateKeyEx(%s)", longkey);
 	    retval = -4;
 	}
 
@@ -266,11 +323,12 @@ static int ap_registry_create_apache_key(void)
 	    break;
 	}
 
+    free(key);
 	hKey = hKeyNext;
 	index++;
     }
 
-    if (keys[index] == NULL) {
+    if (!key) {
 	/* Close the final key we opened, if we walked the entire
 	 * tree
 	 */
@@ -283,6 +341,8 @@ static int ap_registry_create_apache_key(void)
 	    }
 	}
     }
+    else
+        free(key);
 
     return retval;
 }
@@ -302,14 +362,14 @@ static int ap_registry_create_apache_key(void)
  * logged via aplog_error().
  */
 
-static int ap_registry_store_key_int(char *key, DWORD type, void *value, int value_size)
+static int ap_registry_store_key_int(char *key, char *name, DWORD type, void *value, int value_size)
 {
     long rv;
     HKEY hKey;
     int retval;
 
     rv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		      REGKEY,
+		      key,
 		      0,
 	 	      KEY_WRITE,
 		      &hKey);
@@ -317,7 +377,7 @@ static int ap_registry_store_key_int(char *key, DWORD type, void *value, int val
     if (rv == ERROR_FILE_NOT_FOUND) {
 	/* Key could not be opened -- try to create it 
 	 */
-	if (ap_registry_create_apache_key() < 0) {
+        if (ap_registry_create_key(key) < 0) {
 	    /* Creation failed (error already reported) */
 	    return -4;
 	}
@@ -325,28 +385,26 @@ static int ap_registry_store_key_int(char *key, DWORD type, void *value, int val
 	/* Now it has been created we should be able to open it
 	 */
 	rv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-		  REGKEY,
+		  key,
 		  0,
 	 	  KEY_WRITE,
 		  &hKey);
 
 	if (rv == ERROR_FILE_NOT_FOUND) {
-	    ap_log_error(APLOG_MARK,APLOG_WARNING|APLOG_NOERRNO,NULL,
-		"Registry does not contain key " REGKEY " after creation");
-
+            ap_log_error(APLOG_MARK,APLOG_WARNING|APLOG_NOERRNO,NULL,
+                         "Registry does not contain key %s after creation",key);
 	    return -1;
 	}
     }
 
     if (rv != ERROR_SUCCESS) {
-	do_error(rv, "RegOpenKeyEx HKLM\\" REGKEY,
-		 NULL);
-	return -4;
+        do_error(rv, "RegOpenKeyEx HKLM\\%s", key);
+        return -4;
     }
 
     /* Now set the value and data */
     rv = RegSetValueEx(hKey, 
-		       key,	/* value key name */
+                       name,	/* value key name */
 		       0,	/* reserved */
 		       type,	/* type */
 		       value,	/* value data */
@@ -369,16 +427,33 @@ static int ap_registry_store_key_int(char *key, DWORD type, void *value, int val
      */
     rv = RegCloseKey(hKey);
     if (rv != ERROR_SUCCESS) {
-	do_error(rv, "RegCloseKey HKLM\\" REGKEY, NULL);
-	if (retval == 0) {
-	    /* Keep error status from RegQueryValueEx, if any */
-	    retval = -4;  
-	}
+        do_error(rv, "RegCloseKey HKLM\\%s", key);
+        if (retval == 0) {
+            /* Keep error status from RegQueryValueEx, if any */
+            retval = -4;  
+        }
     }
 
     return retval;
 }
 
+/*
+ * Sets the service confpath value within the registry. Returns 0 on success
+ * or -1 on error. If -1 is return the error will already have been
+ * logged via aplog_error().
+ */
+
+int ap_registry_set_service_conf(char *conf, char *service_name)
+{
+    int rv;
+    char *key = ap_get_service_key(service_name);
+    
+    rv = ap_registry_store_key_int(key, "ConfPath", REG_SZ, conf, strlen(conf)+1);
+    free(key);
+
+    return rv < 0 ? -1: 0;
+}
+
 /*
  * Sets the serverroot value within the registry. Returns 0 on success
  * or -1 on error. If -1 is return the error will already have been
@@ -389,7 +464,8 @@ int ap_registry_set_server_root(char *dir)
 {
     int rv;
 
-    rv = ap_registry_store_key_int("ServerRoot", REG_SZ, dir, strlen(dir)+1);
+    rv = ap_registry_store_key_int(REGKEY, "ServerRoot", REG_SZ, dir, strlen(dir)+1);
 
     return rv < 0 ? -1 : 0;
 }
+
