@@ -75,6 +75,7 @@
  *  4: Failure; operation interrupted (such as with CTRL/C)
  *  5: Failure; buffer would overflow (username, filename, or computed
  *     record too long)
+ *  6: Failure; username contains illegal or reserved characters
  */
 
 #include "ap_config.h"
@@ -83,6 +84,7 @@
 #include <errno.h>
 #include "ap.h"
 #include "ap_md5.h"
+#include "ap_sha1.h"
 
 #ifdef WIN32
 #include <conio.h>
@@ -99,14 +101,17 @@
 #endif /*CHARSET_EBCDIC*/
 
 #define MAX_STRING_LEN 256
+#define ALG_PLAIN 0
 #define ALG_CRYPT 1
 #define ALG_APMD5 2
+#define ALG_APSHA 3 
 
 #define ERR_FILEPERM 1
 #define ERR_SYNTAX 2
 #define ERR_PWMISMATCH 3
 #define ERR_INTERRUPTED 4
 #define ERR_OVERFLOW 5
+#define ERR_BADUSER 6
 
 /*
  * This needs to be declared statically so the signal handler can
@@ -115,17 +120,9 @@
 static char *tempfilename;
 
 /*
- * Duplicate a string into memory malloc()ed for it.
+ * Get a line of input from the user, not including any terminating
+ * newline.
  */
-static char *strd(char *s)
-{
-    char *d;
-
-    d = (char *) malloc(strlen(s) + 1);
-    strcpy(d, s);
-    return (d);
-}
-
 static int getline(char *s, int n, FILE *f)
 {
     register int i = 0;
@@ -155,109 +152,69 @@ static void putline(FILE *f, char *l)
     fputc('\n', f);
 }
 
-
-/* From local_passwd.c (C) Regents of Univ. of California blah blah */
-static unsigned char itoa64[] =	/* 0 ... 63 => ascii - 64 */
-    "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
-
-static void to64(register char *s, register long v, register int n)
-{
-    while (--n >= 0) {
-	*s++ = itoa64[v & 0x3f];
-	v >>= 6;
-    }
-}
-
-#ifdef MPE
-/*
- * MPE lacks getpass() and a way to suppress stdin echo.  So for now, just
- * issue the prompt and read the results with echo.  (Ugh).
- */
-
-static char *getpass(const char *prompt)
-{
-    static char password[81];
-
-    fputs(prompt, stderr);
-    gets((char *) &password);
-
-    if (strlen((char *) &password) > 8) {
-	password[8] = '\0';
-    }
-
-    return (char *) &password;
-}
-
-#endif
-
-#ifdef WIN32
-/*
- * Windows lacks getpass().  So we'll re-implement it here.
- */
-
-static char *getpass(const char *prompt)
-{
-    static char password[81];
-    int n = 0;
-
-    fputs(prompt, stderr);
-    
-    while ((password[n] = _getch()) != '\r') {
-        if (password[n] >= ' ' && password[n] <= '~') {
-            n++;
-            printf("*");
-        }
-	else {
-            printf("\n");
-            fputs(prompt, stderr);
-            n = 0;
-        }
-    }
- 
-    password[n] = '\0';
-    printf("\n");
-
-    if (n > 8) {
-        password[8] = '\0';
-    }
-
-    return (char *) &password;
-}
-#endif
-
 /*
  * Make a password record from the given information.  A zero return
  * indicates success; failure means that the output buffer contains an
  * error message instead.
  */
-static int mkrecord(char *user, char *record, size_t rlen, int alg)
+static int mkrecord(char *user, char *record, size_t rlen, char *passwd,
+		    int alg)
 {
     char *pw;
     char cpw[120];
+    char pwin[MAX_STRING_LEN];
+    char pwv[MAX_STRING_LEN];
     char salt[9];
 
-    pw = strd((char *) getpass("New password: "));
-    if (strcmp(pw, (char *) getpass("Re-type new password: "))) {
-	ap_cpystrn(record, "password verification error", (rlen - 1));
-	return ERR_PWMISMATCH;
+    if (passwd != NULL) {
+	pw = passwd;
+    }
+    else {
+	if (ap_getpass("New password: ", pwin, sizeof(pwin)) != 0) {
+	    ap_snprintf(record, (rlen - 1), "password too long (>%d)",
+			sizeof(pwin) - 1);
+	    return ERR_OVERFLOW;
+	}
+	ap_getpass("Re-type new password: ", pwv, sizeof(pwv));
+	if (strcmp(pwin, pwv) != 0) {
+	    ap_cpystrn(record, "password verification error", (rlen - 1));
+	    return ERR_PWMISMATCH;
+	}
+	pw = pwin;
+        memset(pwv, '\0', sizeof(pwin));
     }
-    (void) srand((int) time((time_t *) NULL));
-    to64(&salt[0], rand(), 8);
-    salt[8] = '\0';
-
     switch (alg) {
-    case ALG_APMD5:
-	ap_MD5Encode(pw, salt, cpw, sizeof(cpw));
+
+    case ALG_APSHA:
+	/* XXX cpw >= 28 + strlen(sha1) chars - fixed len SHA */
+ 	ap_sha1_base64(pw,strlen(pw),cpw);
 	break;
+
+    case ALG_APMD5: 
+        (void) srand((int) time((time_t *) NULL));
+        ap_to64(&salt[0], rand(), 8);
+        salt[8] = '\0';
+
+	ap_MD5Encode((const unsigned char *)pw, (const unsigned char *)salt,
+		     cpw, sizeof(cpw));
+	break;
+
+    case ALG_PLAIN:
+	/* XXX this len limitation is not in sync with any HTTPd len. */
+	ap_cpystrn(cpw,pw,sizeof(cpw));
+	break;
+
     case ALG_CRYPT:
+    default:
+        (void) srand((int) time((time_t *) NULL));
+        ap_to64(&salt[0], rand(), 8);
+        salt[8] = '\0';
+
 	ap_cpystrn(cpw, (char *)crypt(pw, salt), sizeof(cpw) - 1);
 	break;
     }
-    /*
-     * Now that we have the smashed password, we don't need the
-     * plaintext one any more.
-     */
-    free(pw);
+    memset(pw, '\0', strlen(pw));
+
     /*
      * Check to see if the buffer is large enough to hold the username,
      * hash, and delimiters.
@@ -274,10 +231,28 @@ static int mkrecord(char *user, char *record, size_t rlen, int alg)
 
 static int usage(void)
 {
-    fprintf(stderr, "Usage: htpasswd [-cm] passwordfile username\n");
-    fprintf(stderr, "The -c flag creates a new file.\n");
-    fprintf(stderr, "The -m flag forces MD5 encryption of the password.\n");
-    fprintf(stderr, "On Windows systems the -m flag is used by default.\n");
+    fprintf(stderr, "Usage:\n");
+    fprintf(stderr, "\thtpasswd [-cmdps] passwordfile username\n");
+    fprintf(stderr, "\thtpasswd -b[cmdps] passwordfile username password\n\n");
+    fprintf(stderr, " -c  Create a new file.\n");
+    fprintf(stderr, " -m  Force MD5 encryption of the password"
+#if defined(WIN32) || defined(TPF)
+	" (default)"
+#endif
+	".\n");
+    fprintf(stderr, " -d  Force CRYPT encryption of the password"
+#if (!(defined(WIN32) || defined(TPF)))
+	    " (default)"
+#endif
+	    ".\n");
+    fprintf(stderr, " -p  Do not encrypt the password (plaintext).\n");
+    fprintf(stderr, " -s  Force SHA encryption of the password.\n");
+    fprintf(stderr, " -b  Use the password from the command line rather "
+	    "than prompting for it.\n");
+    fprintf(stderr,
+	    "On Windows and TPF systems the '-m' flag is used by default.\n");
+    fprintf(stderr,
+	    "On all other systems, the '-p' flag will probably not work.\n");
     return ERR_SYNTAX;
 }
 
@@ -364,6 +339,7 @@ int main(int argc, char *argv[])
     FILE *ftemp = NULL;
     FILE *fpw = NULL;
     char user[MAX_STRING_LEN];
+    char password[MAX_STRING_LEN];
     char record[MAX_STRING_LEN];
     char line[MAX_STRING_LEN];
     char pwfilename[MAX_STRING_LEN];
@@ -371,7 +347,9 @@ int main(int argc, char *argv[])
     int found = 0;
     int alg = ALG_CRYPT;
     int newfile = 0;
+    int noninteractive = 0;
     int i;
+    int args_left = 2;
 
     tempfilename = NULL;
     signal(SIGINT, (void (*)(int)) interrupted);
@@ -401,6 +379,19 @@ int main(int argc, char *argv[])
 	    else if (*arg == 'm') {
 		alg = ALG_APMD5;
 	    }
+	    else if (*arg == 's') {
+		alg = ALG_APSHA;
+	    }
+	    else if (*arg == 'p') {
+		alg = ALG_PLAIN;
+	    }
+	    else if (*arg == 'd') {
+		alg = ALG_CRYPT;
+	    }
+	    else if (*arg == 'b') {
+		noninteractive++;
+		args_left++;
+	    }
 	    else {
 		return usage();
 	    }
@@ -408,10 +399,11 @@ int main(int argc, char *argv[])
     }
 
     /*
-     * Make sure we still have exactly two arguments left (the filename
-     * and the username).
+     * Make sure we still have exactly the right number of arguments left
+     * (the filename, the username, and possibly the password if -b was
+     * specified).
      */
-    if ((argc - i) != 2) {
+    if ((argc - i) != args_left) {
 	return usage();
     }
     if (strlen(argv[i]) > (sizeof(pwfilename) - 1)) {
@@ -420,10 +412,24 @@ int main(int argc, char *argv[])
     }
     strcpy(pwfilename, argv[i]);
     if (strlen(argv[i + 1]) > (sizeof(user) - 1)) {
-	fprintf(stderr, "%s: username too long\n", argv[0]);
+	fprintf(stderr, "%s: username too long (>%d)\n", argv[0],
+		sizeof(user) - 1);
 	return ERR_OVERFLOW;
     }
     strcpy(user, argv[i + 1]);
+    if ((arg = strchr(user, ':')) != NULL) {
+	fprintf(stderr, "%s: username contains illegal character '%c'\n",
+		argv[0], *arg);
+	return ERR_BADUSER;
+    }
+    if (noninteractive) {
+	if (strlen(argv[i + 2]) > (sizeof(password) - 1)) {
+	    fprintf(stderr, "%s: password too long (>%d)\n", argv[0],
+		    sizeof(password) - 1);
+	    return ERR_OVERFLOW;
+	}
+	strcpy(password, argv[i + 2]);
+    }
 
 #ifdef WIN32
     if (alg == ALG_CRYPT) {
@@ -432,6 +438,12 @@ int main(int argc, char *argv[])
     }
 #endif
 
+#if (!(defined(WIN32) || defined(TPF)))
+    if (alg == ALG_PLAIN) {
+	fprintf(stderr,"Warning: storing passwords as plain text might "
+		"just not work on this platform.\n");
+    }
+#endif
     /*
      * Verify that the file exists if -c was omitted.  We give a special
      * message if it doesn't.
@@ -480,7 +492,10 @@ int main(int argc, char *argv[])
      * Any error message text is returned in the record buffer, since
      * the mkrecord() routine doesn't have access to argv[].
      */
-    if ((i = mkrecord(user, record, sizeof(record) - 1, alg)) != 0) {
+    i = mkrecord(user, record, sizeof(record) - 1,
+		 noninteractive ? password : NULL,
+		 alg);
+    if (i != 0) {
 	fprintf(stderr, "%s: %s\n", argv[0], record);
 	exit(i);
     }
